/*
 * #%L
 * JRst :: Api
 * 
 * $Id: ImageDirective.java 730 2012-06-29 23:56:47Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/jrst/tags/jrst-2.0.1/jrst/src/main/java/org/nuiton/jrst/legacy/directive/ImageDirective.java $
 * %%
 * Copyright (C) 2004 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

package org.nuiton.jrst.legacy.directive;

import org.nuiton.jrst.JRSTDirective;
import org.nuiton.jrst.legacy.JRSTLexer;
import static org.nuiton.jrst.legacy.ReStructuredText.IMAGE;
import static org.nuiton.jrst.legacy.ReStructuredText.SUBSTITUTION_DEFINITION;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * .. image:: picture.jpeg :height: 100 :width: 200 :scale: 50 :alt: alternate
 * text :align: right
 * 
 * Created: 4 nov. 06 12:52:02
 *
 * @author poussin
 * @version $Revision: 730 $
 *
 * Last update: $Date: 2012-06-30 01:56:47 +0200 (Sat, 30 Jun 2012) $
 * by : $Author: tchemit $
 */
public class ImageDirective implements JRSTDirective {

    protected static final String SCALE = "scale";
    protected static final String WIDTH = "width";
    protected static final String HEIGHT = "height";

    /*
     * (non-Javadoc)
     * 
     * @see org.nuiton.jrst.JRSTDirective#parse(org.dom4j.Element)
     */
    @Override
    public Node parse(Element e) {
        Element result = DocumentHelper.createElement(IMAGE);

        if (e.getParent() != null
                && SUBSTITUTION_DEFINITION.equals(e.getParent().getName())) {
            String ref = e.getParent().attributeValue("name");
            result.addAttribute("alt", ref);
        }
        result.addAttribute("uri", e.attributeValue(JRSTLexer.DIRECTIVE_VALUE));

        Pattern arg = Pattern.compile(":([^:]+):\\s*(.*)");
        String[] lines = e.getText().split("\n");
        for (String l : lines) {
            Matcher matcher = arg.matcher(l.trim());
            if (matcher.matches()) {
                String name = matcher.group(1);
                String value = matcher.group(2);
                if (SCALE.equalsIgnoreCase(name)) {
                    if (!result.asXML().matches(".*" + WIDTH + "=\".*\".*")) {
                        result.addAttribute(WIDTH, value + (value.matches(".*%") ? "" : "%"));
                    }
                    if (!result.asXML().matches(".*" + HEIGHT + "=\".*\".*")) {
                        result.addAttribute(HEIGHT, value + (value.matches(".*%") ? "" : "%"));
                    }
                }
                result.addAttribute(name, value);
            }
        }
        return result;
    }

}
